/*
  Copyright (C) 2004, 2005, 2008 Rocky Bernstein <rocky@gnu.org>
  Copyright (C) 2014 Robert Kausch <robert.kausch@freac.org>
  Copyright (C) 2001 Monty <xiphmont@mit.edu>
*/
/******************************************************************
 * cdda_paranoia generation III release 10.2 libcdio
 ******************************************************************/
#include <cdio/version.h>

#define PARANOIA_VERSION \
  "cdparanoia III release 10.2 libcdio " CDIO_VERSION "\n"		\
  "(C) 2001 Monty <monty@xiph.org> and Xiphophorus\n"			\
  "(C) 2004, 2005, 2008 Rocky Bernstein <rocky@gnu.org>\n"		\
  "(C) 2014 Robert Kausch <robert.kausch@freac.org>\n\n"		\
  "Report bugs to bug-libcdio@gnu.org\n"
